
IDENTIFIED_COMMANDS = {
    'NS STATUS %s': ['STATUS {username} (\d)', '3'],
    'PRIVMSG NickServ ACC %s': ['{username} ACC (\d)', '3'],
}

IRC_AUTHS = {
    # 'localhost': {'username': 'Botname', 'realname': 'Bot Real Name'},
}

IRC_GROUPCHATS = [
    # 'groupchat@localhost',
]

IRC_PERMISSIONS = {
    # 'nekmo@localhost': ['root'],
}